<?php
namespace Zeedhi\Tools\Generator;

use Seld\JsonLint\ParsingException;
use Zeedhi\Tools\EntityGenerator\Inflector;
use Seld\JsonLint\JsonParser;

class GeneratorConfig {

    /** @var string */
    protected $mappingPath;
    /** @var string */
    protected $srcPath;
    /** @var ConnectionConfig */
    protected $connectionConfig;
    /** @var array */
    protected $subModelsByTable;
    /** @var array */
    protected $inverseRelations;
    /** @var string */
    protected $namespace;
    /** @var string */
    protected $mappingType;
    /** @var string */
    protected $inflectorClassName;
    /** @var array */
    protected $dbReaderExtraConfig = array();
    /** @var string */
    protected $psr4Root = "";

    /**
     * @param $mappingPath
     * @param $srcPath
     * @param $namespace
     * @param $connectionConfig
     * @param $subModelsByTable
     * @param $inverseRelations
     * @param $mappingType
     * @param $inflectorClassName
     * @param $psr4Root
     */
    function __construct(
        $mappingPath,
        $srcPath,
        $namespace,
        $connectionConfig,
        $subModelsByTable,
        $inverseRelations,
        $mappingType,
        $inflectorClassName
    ) {
        $this->mappingPath = $mappingPath;
        $this->namespace = $namespace;
        $this->srcPath = $srcPath;
        $this->subModelsByTable = $subModelsByTable;
        $this->inverseRelations = $inverseRelations;
        $this->connectionConfig = $connectionConfig;
        $this->mappingType = $mappingType;
        $this->inflectorClassName = $inflectorClassName;
    }

    /**
     * @param $configArray
     * @return mixed
     * @throws Exception
     */
    protected static function validateConfigParameters($configArray) {
        $parametersToValidate = array(
            "mappingPath",
            "srcPath",
            "namespace",
            "connection",
            "mappingType",
            "inflectorClassName",
            "subModelsByTables"
        );

        foreach ($parametersToValidate as $parameterName) {
            if (!isset($configArray[$parameterName])) {
                throw Exception::missingParameterInConfig($parameterName);
            }
        }
        return $configArray;
    }

    /**
     * Factory a instance of GeneratorConfig from a given array.
     *
     * @param array $configArray
     *
     * @throws Exception When config array missing any parameter.
     *
     * @return GeneratorConfig
     */
    public static function factoryFromArray($configArray) {
        $subModelsByTable = isset($configArray["subModelsByTables"]) ? $configArray["subModelsByTables"] : array();
        $inverseRelations = isset($configArray["inverseRelations"]) ? $configArray["inverseRelations"] : array();
        $configArray = self::validateConfigParameters($configArray);
        $instance = new static(
            $configArray["mappingPath"],
            $configArray["srcPath"],
            $configArray["namespace"],
            ConnectionConfig::factoryFromArray($configArray['connection']),
            $subModelsByTable,
            $inverseRelations,
            $configArray["mappingType"],
            $configArray["inflectorClassName"]
        );

        if (isset($configArray['dbReaderExtraConfig'])) {
            $instance->setDbReaderExtraConfig($configArray['dbReaderExtraConfig']);
        }

        if (isset($configArray['psr4Root'])) {
            $instance->setPsr4Root($configArray['psr4Root']);
        }

        return $instance;
    }

    /**
     * Factory a instance of GeneratorConfig from a given string.
     *
     * @param string $configString Given string.
     *
     * @throws ParsingException
     *
     * @return GeneratorConfig
     */
    public static function factoryFromJSONString($configString) {
        $parser = new JsonParser();
        if ($result = $parser->lint($configString)) {
            throw $result;
        }

        return self::factoryFromArray(json_decode($configString, true));
    }

    /**
     * @return string
     */
    public function getMappingPath() {
        return $this->mappingPath;
    }

    /**
     * @return string
     */
    public function getSrcPath() {
        return $this->srcPath;
    }

    /**
     * @return array
     */
    public function getSubModelsByTable() {
        return $this->subModelsByTable;
    }

    /**
     * @return array
     */
    public function getInverseRelations() {
        return $this->inverseRelations;
    }

    /**
     * @return ConnectionConfig
     */
    public function getconnectionConfig() {
        return $this->connectionConfig;
    }

    /**
     * @return string
     */
    public function getNamespace() {
        return $this->namespace;
    }

    /**
     * @return string
     */
    public function getMappingType() {
        return $this->mappingType;
    }

    /**
     * @return Inflector
     */
    public function getInflector() {
        $className = $this->inflectorClassName;
        return new $className();
    }

    /**
     * @return array
     */
    public function getDbReaderExtraConfig() {
        return $this->dbReaderExtraConfig;
    }

    /**
     * @param array $dbReaderExtraConfig
     */
    public function setDbReaderExtraConfig($dbReaderExtraConfig) {
        $this->dbReaderExtraConfig = $dbReaderExtraConfig;
    }

    /**
     * @param $psr4Root
     */
    public function setPsr4Root($psr4Root) {
        $this->psr4Root = $psr4Root;
    }

    /**
     * @return string
     */
    public function getPsr4Root() {
        return $this->psr4Root;
    }
}